%{

/* 
 * (was file: fmt.c)
 * 04/88  bgray
 * file: xfmt.l
 * revised with (f)lex front end
 * 05/88  Greg Lee
 */

/* a simple text formatter */

#ifndef lint
static	char xfmt_sccsid[] = "@(#)xfmt.l <05/30/88>";
#endif lint


#include <stdio.h>
#include <ctype.h>
#include <string.h>

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/* character attributes controlled by font-changing commands */
#define ROMAN  1	/* plain, from \rm or .R */
#define ITALIC 2	/* underlined, from \it or .I */
#define BOLD   3	/* bold, from \bf or .B */
#define TTYPE  4	/* reversed, from \tt or .S */
#define SLANTY 5	/* blinking, from \sl or .G */

int     attribute = ROMAN,	/* current character attribute */
        nextattribute = 0;	/* attribute to restore after next
				 * word for .RB, etc. or after line
				 * for \beginsection
				 */
char    nest[80];		/* stack of attributes for each current */
int	level = 1;		/* TeX {}-group */

/* global state of formatter */
char    iflag  = FALSE,		/* preserving left indentation (-i)? */
        jflag  = FALSE,		/* requested justification with -j? */
        oflag  = FALSE,		/* requested overprint with -o? */
        uflag  = FALSE,		/* requested terminal output with -u? */
        xflag  = FALSE,		/* requested use TeX commands w. -x? */
        mflag  = FALSE,		/* requested use man commands w. -m? */
        cflag  = FALSE,		/* requested format C code w. -c? */
        qtflag = FALSE,		/* saw odd " in processing .RB etc.? */
        ceflag = FALSE,		/* center next output line? */
        raflag = FALSE,		/* right adjust next output line? */
        obflag = FALSE,		/* not concatenating? */
        njflag = FALSE;		/* not currently justifying? */

int     maxlen  = 72,
        tabsize = 8,
        indent  = 0,
        offset  = 0,
        tpstate = 0,		/* keep track for hanging indentation */
        tpvalue = 5,		/* amount of hang, from arg of .TP,.HP */
        spaces  = 0,		/* space to leave after current word */
        tempi;			/* misc. use in lex pattern section */

char   *progname = NULL;
char   *usage = "usage: %s [-ijcmxou] [-l n] [-p n] [-t n] [file ...]\n";

int     dir = FALSE;
int     holecnt = 0;
char   *holeptr[256];		/* holeptr[0] is unused */

char	oline[2048] = "\0",	/* output line buffer */
       *olp = oline,
        aline[2048] = "\0",	/* output line attribute buffer */
       *alp = aline,
        oword[512] = "\0",	/* output word buffer */
       *owp = oword,
        aword[512] = "\0",	/* output word attribute buffer */
       *awp = aword;

extern char *optarg;		/* from getopt */
extern int  optind;

int     max ();
void    exit ();
char   *basename ();
void    copyindent ();
void    puttc ();
void    putts ();
void    lbreak ();
void    putword ();
void    justifyline ();
void    putline ();
void    sputline ();
void    skipline ();
void    addparens ();
void    terminit ();
void    termattr ();

%}

/* lex start state for flags: -x, -m, (none), -mx, respectively */
%s	TEX MAN PLAIN ALL CCODE

/* match end of TeX command */
wh	[ \t]*/[^a-z]

%%

<MAN,ALL>^".sp"		skipline(1);

/* match various TeX skip commands */
<TEX,ALL>\\[a-gi-z]+"skip"{wh}	skipline(1);

/* blank line in input gives blank line in output -- not like real TeX */
\n$			skipline(1);

<MAN,ALL>^".sp "[0-9]+	skipline(atoi(yytext+4));

/* traditional */
<PLAIN>^\..* {
			lbreak();
			(void)printf("%s\n",yytext);
}

/* escaped special characters -- \\ for \ not like real TeX */
<TEX,ALL>\\[$#{}&%^_~'"\\] puttc(yytext[1]);

/* some font-changing commands */

<TEX,ALL>"\\it"{wh} |
<MAN,ALL>"\\f"("I"|"C")	attribute = ITALIC;

<TEX,ALL>"\\sl"{wh} |
<MAN,ALL>"\\fG"     	attribute = SLANTY;

<TEX,ALL>"\\bf"{wh} |
<MAN,ALL>"\\fB"     	attribute = BOLD;

<TEX,ALL>"\\rm"{wh} |
<MAN,ALL>"\\f"("R"|"P")	attribute = ROMAN; /* \fP should not be like \fR */

<TEX,ALL>"\\tt"{wh} |
<MAN,ALL>"\\fS"     	attribute = TTYPE;


<TEX,ALL>"\\TeX"{wh}	putts("TeX");

/* should these 4 TeX commands cause breaks? */
<TEX,ALL>\\"centerline"{wh}	ceflag = TRUE;

<TEX,ALL>\\"rightline"{wh}	raflag = TRUE;

<TEX,ALL>\\"raggedright"{wh}	njflag = level;

<TEX,ALL>\\"obeylines"{wh}	obflag = level;

<MAN,ALL>^".nf"		lbreak(); obflag = TRUE;

<MAN,ALL>^".fi"		lbreak(); obflag = FALSE;

<TEX,ALL>"{"		nest[level++] = attribute; /* push attr. onto stack */

<TEX,ALL>"}"  {		/* if emerging from {} in which \obeylines or
			 * \raggedright occurred, cancel  (not a very general
			 * strategy)
			 */
			if (obflag == level) obflag = FALSE;
			if (njflag == level) njflag = FALSE;
			/* pop attribute off stack */
			if (level > 1) attribute = nest[--level];
			/* } after \item signals end of tag; will not
			 * work right with {} inside tag */
			if (tpstate == 1)  {
				tpstate = 2;
				putword();
			}
}


^[ \t]+ {
			putword();
			if (iflag || obflag) {
				putline();
				tempi = attribute;
				/* for comments with indentation */
				if (cflag) attribute = ROMAN;
				copyindent(yytext);
				attribute = tempi;
			}
			else if (mflag)
				putline();
}

<TEX,ALL>"\\ "		putword();

<MAN,ALL,CCODE>\n  {	/* for C code reverse all of multiline comments */
			if (!cflag) attribute = ROMAN;
			nextattribute = qtflag = FALSE;
			/* signal end of tag for hanging indent */
			if (tpstate == 1) tpstate = 2;
			putword();
			if (obflag) putline();
}

/* more font-changing */

<MAN,ALL>^\.B" "	attribute = BOLD;

<MAN,ALL>^\.(I|C)" "	attribute = ITALIC;

<MAN,ALL>^\.R" "	attribute = ROMAN;

<MAN,ALL>^\.("S"|"SM")" "	attribute = TTYPE;

<MAN,ALL>^\.G" "	attribute = SLANTY;

<MAN,ALL>\\[\|\^&!{}\nacdeprtuz]		;

<MAN,ALL>\\.		puttc(yytext[1]);

<MAN,ALL>^".ti +"[0-9]+  {
	 		lbreak();
			for (tempi = 0; tempi < atoi(yytext+5) - 1; tempi++)
				puttc(' ');
}

<MAN,ALL>^\.br 		lbreak();

<MAN,ALL>^\.(P|L|"")P 	skipline(1); indent = 5;

/* hanging indent stuff (awfully complicated) */

<MAN,ALL>^\.(T|H)"P "[0-9]+\n  {
			skipline(1);
			if (yytext[1] == 'H')
				tpstate = 4;
			else	tpstate = 1;
			tpvalue = atoi(yytext+4);
			indent = 5;
}

<TEX,ALL>\\"item"("item")?{wh}  |
<MAN,ALL>^".HP ".*\n	|
<MAN,ALL>^".TP".*\n	|
<MAN,ALL>^".IP"[ \t]*\n?  {
			skipline(1);
			if (yytext[1] == 'I' && yytext[3] == '\n')
				tpstate = 4;
			else	tpstate = 1;
			tpvalue = 5;
			indent = 5;
}

/* various begin-end commands to set off blocks of text */

<MAN,ALL>^"."(R|D)(S|E)  {
			if (yytext[1] == 'D' && yytext[2] == 'S') {
				skipline(1);
				obflag = TRUE;
			}
			else	lbreak();
			indent = 10;
			if (yytext[2] == 'E') {
				indent = 5;
				obflag = FALSE;
			}
}

<MAN,ALL>^".E"(X|E)  {
			skipline(1);
			if (yytext[2] == 'E') {
				indent = 5;
				obflag = FALSE;
			}
			else {	indent = 10;
				obflag = TRUE;
			}
}


<MAN,ALL>^".N"(T|E).*  {
			skipline(1);
			if (yytext[2] == 'E') {
				indent = 5;
				maxlen += 5;
			}
			else {	indent = 10;
				maxlen -= 5;
				tempi = attribute;
				attribute = ITALIC;
				ceflag = TRUE;
				if (yyleng < 5) putts("NOTE");
				else putts(yytext+4);
				skipline(1);
			}
}

/* those curious -man commands to join words, alternating fonts */

<MAN,ALL>^\.("IR"|"PN")" "	nextattribute = ROMAN; attribute = ITALIC;

<MAN,ALL>^\."BR "	nextattribute = ROMAN; attribute = BOLD;

<MAN,ALL>^\."RI "	nextattribute = ITALIC; attribute = ROMAN;

<MAN,ALL>^\."BI "	nextattribute = ITALIC; attribute = BOLD;

<MAN,ALL>^\."RB "	nextattribute = BOLD; attribute = ROMAN;

<MAN,ALL>^\."IB "	nextattribute = BOLD; attribute = ITALIC;

/* section headings */

<TEX,ALL>\\"beginsection"{wh}  {
			skipline(1);
			nextattribute = attribute;
			attribute = BOLD;
}

<MAN,ALL>^\."S"(H|S)" ".* {
			skipline(1);
			if (yytext[2] == 'H')
				indent = 0;
			attribute = BOLD;
			putts(yytext+4);
			lbreak();
			indent = 5;
			attribute = ROMAN;
}

<MAN,ALL>^\."TH ".* {
			lbreak();
			indent = 0;
			addparens(yytext+4, BOLD);
			raflag = TRUE;
			skipline(2);
			indent = 5;
}

/* "Manual Section" reference */
<MAN,ALL>^".MS ".*	addparens(yytext+4, ITALIC);

<MAN,ALL>^".CT ".  {
			if (uflag || oflag) {
				tempi = attribute;
				attribute = TTYPE;
				puttc(toupper(yytext[4]));
				attribute = tempi;
			}
			else {
				putts("<CTRL/");
				puttc(yytext[4]);
				puttc('>');
			}
}

<MAN,ALL>^".V"(S|E).* |
<MAN,ALL>^".UC"(..)?  |
<MAN,ALL>^\.\\\".*	;

/* for join-words commands, words can be quoted */
<MAN,ALL>\"  {
			if (nextattribute || tpstate == 1)  {
				qtflag = !qtflag;
			}
			else puttc('"');
}

<MAN,ALL,CCODE>" "  {
			if (nextattribute)  {
				if (qtflag) puttc(' ');
				else  {
					tempi = attribute;
					attribute = nextattribute;
					nextattribute = tempi;
				}
			}
			else {
				putword();
				if (obflag) copyindent(yytext);
			}
}

<TEX,ALL,CCODE>\n  {
			putword();
			if (obflag || nextattribute) putline();
			if (nextattribute) {
				attribute = nextattribute;
				nextattribute = 0;
			}
}

<TEX,ALL>"~"			puttc(' ');

[ \t]+  {
			putword();
			if (obflag) copyindent(yytext);
}

[\n\r\f]			putword();

<TEX,ALL>\\"backslash"{wh}	puttc('\\');

<TEX>\\    |
<TEX,ALL>\$|\#			;

<TEX,ALL>\\[A-Za-z]+{wh}	;

<MAN,ALL>\\"*(rq"	puttc('\'');

<MAN,ALL>\\"*(lq"	puttc('`');

<MAN,ALL>\\"("..  {
			tempi = attribute;
			attribute = TTYPE;
			if (yytext[2] != '*') puttc(yytext[2]);
			puttc(yytext[3]);
			attribute = tempi;
}

<MAN,ALL>\\"s"("-")?[0-9] |
<MAN,ALL>"\\*S"		|
<MAN,ALL>^".DT"		|
<MAN,ALL>^".ta ".*	|
<MAN,ALL>^".PD".*	|
<MAN,ALL>^".NX".*	;

<CCODE>\"	{	if (attribute != TTYPE) qtflag = !qtflag;
			puttc('"');
}

<CCODE>"/*"	{	puttc('/');
			if (!qtflag) attribute = TTYPE;
			puttc('*');
}

<CCODE>"*/"	{	puttc('*');
			if (!qtflag) attribute = ROMAN;
			puttc('/');
}

/* control flow keywords are bold */
<CCODE>"continue"|"default"|"do"|"goto"|"return"  |
<CCODE>"if"|"else"|"while"|"break"|"switch"|"case"|"for"  {
			if (attribute != TTYPE && !qtflag) {
				attribute = BOLD;
				putts(yytext);
				attribute = ROMAN;
			} else putts(yytext);
}

/* storage class keywords are underlined */
<CCODE>("auto"|"char"|"double"|"enum"|"extern"|"float")  |
<CCODE>("int"|"long"|"register"|"short"|"sizeof"|"static") |
<CCODE>("struct"|"typedef"|"union"|"unsigned"|"void") {

			if (attribute != TTYPE && !qtflag) {
				attribute = ITALIC;
				putts(yytext);
				attribute = ROMAN;
			} else putts(yytext);
}

/* preprocessor keywords are blinking */
<CCODE>"#"[ \t]*("define"|"undef"|"include"|"if"|"ifdef")  |
<CCODE>"#"[ \t]*("ifndef"|"else"|"endif"|"line")  {

			if (attribute != TTYPE && !qtflag) {
				attribute = SLANTY;
				putts(yytext);
				attribute = ROMAN;
			} else putts(yytext);
}

/* identifiers that happen to contain keywords are plain */
<CCODE>[a-zA-Z_]+	putts(yytext);

.			puttc(yytext[0]);



%%

/*
 * Characters, along with the currently active "font"= screen
 * attribute, are collected in a word buffer until we hit a
 * word break (white space), then moved to a line buffer.
 * Words are collected in the line buffer until we hit a line
 * break (end of paragraph) or the line is long enough, then
 * the line is output.
 */

main (argc, argv)
int     argc;
char   *argv[];
{
	int     c, i;

	progname = basename (argv[0]);
	while ((c = getopt (argc, argv, "ijcxmoul:p:t:")) != EOF)
		switch (c) {
			case 'i': iflag = TRUE; break;
			case 'j': jflag = TRUE; break;
			case 'c': cflag = TRUE; break;
			case 'x': xflag = TRUE; break;
			case 'm': mflag = TRUE; break;
			case 'o': oflag = TRUE; break;
			case 'u': uflag = TRUE; break;
			case 'l': maxlen = max (0, atoi (optarg));
				  break;
			case 'p': offset = max (0, atoi (optarg));
				  break;
			case 't': tabsize = max (1, atoi (optarg));
				  break;
			default: (void) fprintf (stderr, usage, progname);
				  exit (1);
				  break;
		}


/* set lex start state */
	if (xflag && mflag) {
		BEGIN (ALL);
	}
	else if (xflag) {
		BEGIN (TEX);
	}
	else if (mflag) {
		indent = 5;
		BEGIN (MAN);
	}
	else if (cflag) {
		obflag = TRUE;
		if (!oflag) uflag = TRUE;
		BEGIN (CCODE);
	}
	else {
		BEGIN (PLAIN);
	}

/* consult termcap for terminal controls */
	if (uflag)
		terminit ();


/* call lex scanner */
	if (optind >= argc) {
		(void) yylex ();
		lbreak ();
	}
	else
		for (; (optind < argc); optind++) {
			if (freopen (argv[optind], "r", yyin) != NULL) {
				/* unstructured -- couldn't find any other
				 * way to get flex to look at > 1 file */
				yy_init = 1;
				(void) yylex ();
				lbreak ();
			}
			else {
				(void) fprintf (stderr,
				    "Couldn't open file: %s", argv[optind]);
				exit (1);
			}
		}

	return (0);
}				/* main */


/* preserve spacing due to spaces or tabs */
void	copyindent (ilp)
char	*ilp;
{
	int     col;

	col = (olp - oline) + 1;

	for (; (isspace (*ilp)); col++) {
		if (*ilp++ == '\t')
			for (; (col % tabsize); col++) {
				*olp++ = ' ';
				*alp++ = (cflag) ? attribute : ROMAN;
			}
		*olp++ = ' ';
		*alp++ = (cflag) ? attribute : ROMAN;
	}
}				/* copyindent */

/* one character goes into word buffer */
void puttc (ch)
char    ch;
{
	*owp++ = ch;
	*awp++ = attribute;
}				/* puttc */

/* put string in word buffer (might have spaces) */
void putts (s)
char   *s;
{
	while (*s)
		puttc (*s++);
}				/* putts */

/* finish pending word and line and put it out (no justification) */
void lbreak () {
	putword ();
	putline ();
	tpstate = 0;
}				/* lbreak */

/* append pending word to line buffer */
void putword () {
	int     plen;
	char   *p;
	char   *q;
	static char     started = 0;

	/* just return if no characters have been accumulated yet */
	if (owp == oword) {
		/* this is for the special case in which a file
		 * starts with \n, which should count as  a "blank line"
		 */
		if (!started) {
			putchar ('\n');
			started = 1;
		}
		return;
	}
	started = 1;

	puttc ('\0');
	p = oword;
	q = aword;
	plen = strlen (p);

	/* ugly way to ignore "0.3i" arg of .IP */
	if (tpstate == 2 && *p == '0' && p[plen - 1] == 'i') {
		spaces = plen = 0;
		*p = '\0';
	}

	/* finish pending line if pending word won't fit */
	if (!obflag && (olp - oline) + spaces + plen > maxlen - indent) {
		if ((jflag) && (holecnt) && !njflag)
			justifyline ();
		putline ();
	}

	/* append spaces which were previously decided to go after
	 * last word put in line buffer */
	if (spaces) {
		/* don't use space inside tag of hanging indent to
		 * right justify */
		if (!tpstate || tpstate > 3)
			holeptr[++holecnt] = olp;
		/* if tag already padded out, permit justification
		 * of space after next word */
		if (tpstate == 3)
			tpstate = 4;
		for (; (spaces > 0); spaces--) {
			*olp++ = ' ';
			/* a matter of taste -- put just " = ROMAN"
			 * for no underlined spaces between words */
			*alp++ = (*(alp-1) == ITALIC && *q == ITALIC) ?
				ITALIC : ROMAN;
		}
	}

	/* figure spaces which should go after pending word which
	 * is just about to be appended in line buffer -- remember
	 * so they can be put before next word (cf. just above) */
	if (!obflag) spaces = 1 + endofsentence (p, plen);
	/* (but if spacing is being preserved, as signaled by
	 *  obflag, we don't want extra spaces) */

	/* append pending word in line buffer */
	while (*p) {
		*olp++ = *p++;
		*alp++ = *q++;
	}

	/* part of tortuous way of handling hanging indent */
		/* tpstate = 0: no hanging indent
		 *	     1: scanning tag to go in left margin
		 *	     2: have seen end of tag, time to
		 *		pad out tag to hanging indent value
		 *	     3: finished padding
		 *	     4: finished first line of hanging par.;
		 *		subsequent lines should be indented
		 *		to hanging indent value
		 */
	if (tpstate == 2) {	/* time to pad tag? */
		tpstate = 3;	/* signal padding done */
		if ((olp - oline) + spaces > tpvalue) /* will tag fit? */
			putline ();	/* no it won't, so start new line */
		else		/* yes it will, so pad it out */
			while ((olp - oline) + spaces < tpvalue)
				spaces++;
	}

	/* purge word buffer -- get ready for next word */
	owp = oword;
	awp = aword;
}				/* putword */


/* increase inter-word spacings for right justification */
void justifyline () {
	int     n;
	char   *fp,
	       *tp,
	       *fa,
	       *ta;

	dir = (!(dir));
	fp = olp - 1;
	fa = alp - 1;
	olp = &oline[maxlen - indent];
	alp = &aline[maxlen - indent];
	tp = olp - 1;
	ta = alp - 1;
	while (tp > fp) {
		while (fp >= holeptr[holecnt]) {
			*tp-- = *fp--;
			*ta-- = *fa--;
		}
		if (dir)
			n = ((tp - fp) - 1) / holecnt + 1;
		else
			n = (tp - fp) / holecnt;
		while (n--) {
			*tp-- = ' ';
			*ta-- = ta[1];
		}
		holecnt--;
	}
}				/* justifyline */


/* put out pending line in line buffer */
void putline ()
{
	if (ceflag) {
		spaces = max ((maxlen - (olp - oline)) / 2, 1) + offset;
		ceflag = FALSE;
	}
	else if (raflag) {
		spaces = max ((maxlen - (olp - oline)), 1) + offset;
		raflag = FALSE;
	}
	else	spaces = indent + offset;

	*olp = '\0';

	if (*oline)
		/* if -u or -x options, have to do it character
		 * by character */
		if (oflag || uflag)
			sputline ();
	else
			(void) printf ("%*s\n", spaces + strlen (oline), oline);

	/* purge line buffer -- get ready for new line */
	*oline = '\0';
	olp = oline;
	alp = aline;
	spaces = 0;
	holecnt = 0;

	/* if finished with first line of hanging par., increase
	 * left indent for subsequent lines */
	if (tpstate > 2) {
		indent = tpvalue + 5;
		tpstate = 4;
	}
}				/* putline */


/* display characters of line with overstriking or with special
 * terminal attributes */
void sputline () {
	char   *p;
	char   *q;
	char    last = ROMAN;

	if (spaces)
		(void) printf ("%*s", spaces, NULL);

	for (p = oline, q = aline; *p; p++, q++)
		if (uflag) {
			last = newattr (last, *q);
			(void) printf ("%c", *p);
		} else
		switch (*q) {
			case ROMAN: 
				(void) printf ("%c", *p);
				break;
			case BOLD: 
			case TTYPE: 
				if (*p == ' ')
					printf("%c", *p);
				else	(void) printf ("%c%c%c", *p, 8, *p);
				break;
			case ITALIC: 
			case SLANTY: 
				(void) printf ("_%c%c", 8, *p);
				break;
		}
	if (uflag)
		last = newattr (last, ROMAN);
	(void) printf ("\n");

}				/* sputline */

/* put blank lines */
void skipline (n)
int     n;
{
	lbreak ();
	for (; (n > 0); n--)
		(void) printf ("\n");
	if (xflag)
		indent = 0;
}				/* skipline */


/* decide how many spaces go after word */
int     endofsentence (p, plen)
char   *p;
int     plen;
{
	if (plen < 3)
		return (FALSE);
	if (!strchr (".:?!", *(p + plen - 1)))
		return (FALSE);
	if (abbr (p))
		return (FALSE);
	return (TRUE);
}				/* endofsentence */


/* detect "." which is not real end of sentence */
int     abbr (s)
char   *s;
{
	char   *p,
	       *q,
	       *r;

	while (*s == '(')
		s++;
	q = ".i.e.g.dr.mr.mrs.st.";
	for (; (*q); q++) {
		p = q;
		r = s;
		while ((*r) && (*p++ == (*r++ | 0x20)));
		if (!*r)
			return (TRUE);
	}
	return (FALSE);
}				/* abbr */



char   *basename (s)
char   *s;
{
	char   *p;

	if (p = strrchr (s, '/'))
		return (++p);
	else
		return (s);
}				/* basename */



int     max (a, b)
int     a,
        b;
{
	return ((a > b) ? a : b);
}				/* max */


/* put parentheses around number of manual section */
void addparens (s, a)
char   *s;
int     a;
{
	nextattribute = attribute;
	attribute = a;
	while (*s && *s != ' ')
		puttc (*s++);
	if (*s == ' ')
		s++, puttc ('(');
	while (*s && *s != ' ')
		puttc (*s++);
	if (*s == ' ')
		s++;
	puttc (')');
	attribute = nextattribute;
	nextattribute = 0;
	if (*s && isalpha (*s))
		puttc (' ');
	while (*s)
		puttc (*s++);
}				/* addparens */

/* terminal commands -- accessing routines */

/*
 *  following derived from:
 *  tcap v1.0 Enhanced terminal control program
 *  Author: Eric Lorenzo Lim
 *  Modified by: Becca Thomas and Rik Farrow
 *  (appeared in Nov. '87 Unix World)
 */

#define SO 0
#define SE 1
#define MB 2
#define MD 3
#define MR 4
#define MH 5
#define ME 6
#define US 7
#define UE 8
char   *id[] = {
	"so", "se", "mb", "md", "mr", "mh", "me", "us", "ue",
	(char *) NULL,
};

char   *area[50];
char    bp[1024],
        strbuf[1024];

/* get for later use terminal handling commands we need */
void terminit () {
	int     id_size = 0;
	char   *str,
	       *tmpptr;
	extern char    *getenv (), *tgetstr (), *tgoto ();

	if ((tmpptr = getenv ("TERM")) == (char *) NULL) {
		(void) fprintf (stderr, "xfmt: TERM variable not set\n");
		exit (1);
	}

	str = strbuf;

	if (tgetent (bp, tmpptr))
		while (id[id_size] != (char *) NULL) {
			area[id_size] = tgetstr (id[id_size], &str);
			if (area[id_size] == (char *) NULL)
				area[id_size] = "";
			id_size++;
		}


}				/* terminit */

/* put out a terminal handing command */
void termattr (i)
int     i;
{
	(void) printf ("%s", area[i]);
}				/* termattr */

/* check for change of "font" and send command to terminal
 * when there is a change */
int     newattr (l, a)
int     l,	/* last attribute */
        a;	/* new attribute */
{
	if (l != a) {
		if (l == BOLD || l == TTYPE || l == SLANTY)
			termattr (ME);
		else if (l == ITALIC)
			termattr (UE);

		switch (a) {
			case   BOLD:	termattr (MD); break;
			case ITALIC:	termattr (US); break;
			case  TTYPE:	termattr (MR); break;
			case SLANTY:	termattr (MB); break;
		}
	}
	return (a);
}				/* newattr */

